`timescale 1 ns / 1 ps

module get_data(
	input clk, //24MHz
	input rst_n,
	
	input data,
	input data_empty,
	input data_clk,
	
	input [3:0] buffer_addr,
	
	output reg [7:0] buffer,
	output reg buffer_clk
);

reg [7:0] array [0:7];

always@(buffer_addr)
	if (buffer_addr == 4'd0)
		buffer = 8'hff;
	else if (buffer_addr == 4'd1)
		buffer = 8'haa;
	else
		buffer = array[buffer_addr - 4'd2];


reg [2:0] shift_counter;
reg [7:0] shift_reg;
reg shift_reg_clk;

always @(posedge clk or negedge rst_n)
	if (!rst_n) begin
		shift_counter <= 3'd0;
		shift_reg <= 8'd0;
		shift_reg_clk <= 1'b0;
	end
	else begin
		shift_reg_clk <= 1'b0;
		if (data_clk) begin
			if (data_empty) begin
				shift_counter <= 3'd0;
			end
			else begin
				shift_reg <= {shift_reg[6:0], data};
				shift_counter <= shift_counter + 1'b1;
				if (shift_counter == 3'd7) begin
					shift_reg_clk <= 1'b1;
				end
			end
		end
	end


reg [3:0] state;
parameter SYNC = 4'b0001, PID = 4'b0010, ENDP = 4'b0100, DATA = 4'b1000;
reg data_en;
reg [2:0] data_counter;

always @(posedge clk or negedge rst_n)
	if (!rst_n) begin
		state <= SYNC;
		data_en <= 1'b0;
		data_counter <= 3'd0;
		buffer_clk <= 1'b0;
	end
	else begin
		buffer_clk <= 1'b0;
		
		if (shift_reg_clk) begin
			case (state)
			SYNC: begin
				data_counter <= 3'd0;
				if (shift_reg == 8'b0000_0001) // SYNC
					state <= PID;
				else
					state <= SYNC;
			end
			
			PID: begin
				if (shift_reg == 8'b1001_0110) begin // PID IN
					data_en <= 1'b0;
					state <= ENDP;
				end
				else if (data_en && (shift_reg == 8'b1100_0011 || shift_reg == 8'b1101_0010)) // DATA0 || DATA1
					state <= DATA;
				else begin
					data_en <= 1'b0;
					state <= SYNC;
				end
			end
			
			ENDP: begin
				state <= SYNC;
				if (shift_reg[3:0] == 4'b0001) // ENDP = 1
					data_en <= 1'b1;
				else
					data_en <= 1'b0;
			end
			
			DATA: begin
				data_counter <= data_counter + 1'b1;
				array[data_counter] <= {shift_reg[0],shift_reg[1],shift_reg[2],shift_reg[3],shift_reg[4],shift_reg[5],shift_reg[6],shift_reg[7]};
				if (data_counter == 3'd7) begin
					buffer_clk <= 1'b1;
					state <= SYNC;
				end
			end
			
			default: begin
				state <= SYNC;
			end
			
			endcase
		end
	end

endmodule
